@@ -1,4 +1,5 @@
 from django.core.cache.backends.base import BaseCache, InvalidCacheBackendError
+from django.core.exceptions import ImproperlyConfigured
 from django.utils.encoding import smart_unicode, smart_str
 from django.utils.datastructures import SortedDict
 
@@ -14,6 +15,7 @@ except ImportError:
         "Redis cache backend requires the 'redis-py' library")
 
 
+
 class CacheKey(object):
     """
     A stub string class that we can use to check if a key was created already.
@@ -43,18 +45,19 @@ class CacheClass(BaseCache):
         try:
             db = int(db)
         except (ValueError, TypeError):
-            db = 1
+            raise ImproperlyConfigured("db value must be an integer")
         if ':' in server:
             host, port = server.split(':')
             try:
                 port = int(port)
             except (ValueError, TypeError):
-                port = 6379
+                raise ImproperlyConfigured("port value must be an integer")
         else:
             host = server or 'localhost'
             port = 6379
         self._cache = redis.Redis(host=host, port=port, db=db, password=password)
 
+
     def make_key(self, key, version=None):
         """
         Returns the utf-8 encoded bytestring of the given key as a CacheKey
@@ -87,36 +90,21 @@ class CacheClass(BaseCache):
             return default
         return self.unpickle(value)
 
-    def set(self, key, value, timeout=None, version=None):
+    def set(self, key, value, timeout=None, version=None, client=None):
         """
         Persist a value to the cache, and set an optional expiration time.
         """
+        if not client:
+            client = self._cache
         key = self.make_key(key, version=version)
+        # get timout
+        if not timeout:
+            timeout = self.default_timeout
         # store the pickled value
-        result = self._cache.set(key, pickle.dumps(value))
-        # set expiration if needed
-        self.expire(key, timeout, version=version)
+        result = self._cache.setex(key, pickle.dumps(value), int(timeout))
         # result is a boolean
         return result
 
-    def expire(self, key, timeout=None, version=None):
-        """
-        Set content expiration, if necessary
-        """
-        key = self.make_key(key, version=version)
-        if timeout is None:
-            timeout = self.default_timeout
-        if timeout <= 0:
-            # force the key to be non-volatile
-            result = self._cache.get(key)
-            self._cache.set(key, result)
-        else:
-            # If the expiration command returns false, we need to reset the key
-            # with the new expiration
-            if not self._cache.expire(key, timeout):
-                value = self.get(key, version=version)
-                self.set(key, value, timeout, version=version)
-
     def delete(self, key, version=None):
         """
         Remove a key from the cache.
@@ -149,6 +137,8 @@ class CacheClass(BaseCache):
         """
         Retrieve many keys.
         """
+        if not keys:
+            return {}
         recovered_data = SortedDict()
         new_keys = map(lambda key: self.make_key(key, version=version), keys)
         map_keys = dict(zip(new_keys, keys))
@@ -170,19 +160,11 @@ class CacheClass(BaseCache):
         If timeout is given, that timeout will be used for the key; otherwise
         the default cache timeout will be used.
         """
-        safe_data = {}
+        pipeline = self._cache.pipeline()
         for key, value in data.iteritems():
-            safe_data[key] = pickle.dumps(value)
-        if safe_data:
-            self._cache.mset(dict((self.make_key(key, version=version), value)
-                                   for key, value in safe_data.iteritems()))
-            map(self.expire, safe_data, [timeout]*len(safe_data))
+            self.set(key, value, timeout, version=version, client=pipeline)
+        pipeline.execute()
 
-    def close(self, **kwargs):
-        """
-        Disconnect from the cache.
-        """
-        self._cache.connection.disconnect()
 
 class RedisCache(CacheClass):
     """
